var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "date-fns"], function (require, exports, date_fns_1) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var BaseStrategy = (function () {
        function BaseStrategy(locale) {
            this._previousStrategy = null;
            this._nextStrategy = null;
            this.setLocale(locale);
        }
        BaseStrategy.prototype.setNextStrategy = function (strategy) {
            this._nextStrategy = strategy;
        };
        BaseStrategy.prototype.nextStrategy = function () {
            return this._nextStrategy;
        };
        BaseStrategy.prototype.setPreviousStrategy = function (strategy) {
            this._previousStrategy = strategy;
        };
        BaseStrategy.prototype.previousStrategy = function () {
            return this._previousStrategy;
        };
        BaseStrategy.prototype.isFinal = function () {
            return this._nextStrategy === null;
        };
        BaseStrategy.prototype.setLocale = function (locale) {
            this._locale = locale;
            if (this._locale) {
                this._formattiongOptions = { locale: this._locale };
            }
            else {
                this._formattiongOptions = undefined;
            }
        };
        return BaseStrategy;
    }());
    exports.BaseStrategy = BaseStrategy;
    var YearSelectionStrategy = (function (_super) {
        __extends(YearSelectionStrategy, _super);
        function YearSelectionStrategy() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        YearSelectionStrategy.prototype.next = function (selectedDate) {
            return date_fns_1.addYears(selectedDate, 10);
        };
        YearSelectionStrategy.prototype.previous = function (selectedDate) {
            return date_fns_1.addYears(selectedDate, -10);
        };
        YearSelectionStrategy.prototype.getItems = function (selectedDate) {
            var initialDate = [selectedDate]
                .map(function (date) { return date_fns_1.startOfYear(date); })
                .pop();
            var years = Array.from(new Array(12), function (_, index) { return date_fns_1.addYears(initialDate, index); });
            return years;
        };
        YearSelectionStrategy.prototype.formatDate = function (date) {
            return date_fns_1.format(date, 'YYYY');
        };
        YearSelectionStrategy.prototype.title = function (date) {
            var nextDecade = date_fns_1.addYears(date, 11);
            var start = date_fns_1.format(date, 'YYYY');
            var end = date_fns_1.format(nextDecade, 'YYYY');
            return start + " - " + end;
        };
        YearSelectionStrategy.prototype.isEqual = function (a, b) {
            return date_fns_1.getYear(a) === date_fns_1.getYear(b);
        };
        return YearSelectionStrategy;
    }(BaseStrategy));
    exports.YearSelectionStrategy = YearSelectionStrategy;
    var MonthSelectionStrategy = (function (_super) {
        __extends(MonthSelectionStrategy, _super);
        function MonthSelectionStrategy() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        MonthSelectionStrategy.prototype.next = function (selectedDate) {
            return date_fns_1.addYears(selectedDate, 1);
        };
        MonthSelectionStrategy.prototype.previous = function (selectedDate) {
            return date_fns_1.addYears(selectedDate, -1);
        };
        MonthSelectionStrategy.prototype.getItems = function (selectedDate) {
            var initialDate = [selectedDate]
                .map(function (date) { return date_fns_1.startOfMonth(date); })
                .map(function (date) { return getFirstDayOfWeek(date); })
                .map(function (date) { return date_fns_1.startOfDay(date); })
                .pop();
            var months = Array.from(new Array(12), function (_, index) { return date_fns_1.addMonths(initialDate, index); });
            return months;
        };
        MonthSelectionStrategy.prototype.formatDate = function (date, locale) {
            var options = locale ? { locale: locale } : this._formattiongOptions;
            return date_fns_1.format(date, 'MMM', options);
        };
        MonthSelectionStrategy.prototype.title = function (date) {
            return date_fns_1.format(date, 'YYYY');
        };
        MonthSelectionStrategy.prototype.isEqual = function (a, b) {
            return this.previousStrategy().isEqual(a, b)
                && date_fns_1.getMonth(a) === date_fns_1.getMonth(b);
        };
        return MonthSelectionStrategy;
    }(BaseStrategy));
    exports.MonthSelectionStrategy = MonthSelectionStrategy;
    var DaySelectionStrategy = (function (_super) {
        __extends(DaySelectionStrategy, _super);
        function DaySelectionStrategy() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        DaySelectionStrategy.prototype.next = function (selectedDate) {
            return date_fns_1.addMonths(selectedDate, 1);
        };
        DaySelectionStrategy.prototype.previous = function (selectedDate) {
            return date_fns_1.addMonths(selectedDate, -1);
        };
        DaySelectionStrategy.prototype.getItems = function (selectedDate) {
            var initialDate = [selectedDate]
                .map(function (date) { return date_fns_1.startOfMonth(date); })
                .map(function (date) { return getFirstDayOfWeek(date); })
                .map(function (date) { return date_fns_1.startOfDay(date); })
                .pop();
            var days = Array.from(new Array(42), function (_, index) { return date_fns_1.addDays(initialDate, index); });
            return days;
        };
        DaySelectionStrategy.prototype.formatDate = function (date) {
            return date_fns_1.format(date, 'D');
        };
        DaySelectionStrategy.prototype.title = function (date, locale) {
            var options = locale ? { locale: locale } : this._formattiongOptions;
            return date_fns_1.format(date, 'MMMM YYYY', options);
        };
        DaySelectionStrategy.prototype.isEqual = function (a, b) {
            return this.previousStrategy().isEqual(a, b)
                && date_fns_1.getDate(a) === date_fns_1.getDate(b);
        };
        return DaySelectionStrategy;
    }(BaseStrategy));
    exports.DaySelectionStrategy = DaySelectionStrategy;
    var HourSelectionStrategy = (function (_super) {
        __extends(HourSelectionStrategy, _super);
        function HourSelectionStrategy() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        HourSelectionStrategy.prototype.next = function (selectedDate) {
            return date_fns_1.addHours(selectedDate, 1);
        };
        HourSelectionStrategy.prototype.previous = function (selectedDate) {
            return date_fns_1.addHours(selectedDate, -1);
        };
        HourSelectionStrategy.prototype.getItems = function (selectedDate) {
            var initialDate = [selectedDate]
                .map(function (date) { return date_fns_1.startOfDay(date); })
                .pop();
            var hours = Array.from(new Array(12), function (_, index) { return date_fns_1.addHours(initialDate, index); });
            return hours;
        };
        HourSelectionStrategy.prototype.formatDate = function (date) {
            return date_fns_1.format(date, 'HH');
        };
        HourSelectionStrategy.prototype.title = function (date) {
            return date_fns_1.format(date, 'DD. MM. YYYY');
        };
        HourSelectionStrategy.prototype.isEqual = function (a, b) {
            return this.previousStrategy().isEqual(a, b)
                && date_fns_1.getHours(a) === date_fns_1.getHours(b);
        };
        return HourSelectionStrategy;
    }(BaseStrategy));
    exports.HourSelectionStrategy = HourSelectionStrategy;
    var MinuteSelectionStrategy = (function (_super) {
        __extends(MinuteSelectionStrategy, _super);
        function MinuteSelectionStrategy() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        MinuteSelectionStrategy.prototype.next = function (selectedDate) {
            return date_fns_1.addMinutes(selectedDate, 1);
        };
        MinuteSelectionStrategy.prototype.previous = function (selectedDate) {
            return date_fns_1.addMinutes(selectedDate, -1);
        };
        MinuteSelectionStrategy.prototype.getItems = function (selectedDate) {
            var initialDate = [selectedDate]
                .map(function (date) { return date_fns_1.startOfDay(date); })
                .map(function (date) { return date_fns_1.startOfHour(date); })
                .pop();
            var minutes = Array.from(new Array(12), function (_, index) { return date_fns_1.addMinutes(initialDate, index * 5); });
            return minutes;
        };
        MinuteSelectionStrategy.prototype.formatDate = function (date) {
            return date_fns_1.format(date, 'mm');
        };
        MinuteSelectionStrategy.prototype.title = function (date) {
            return date_fns_1.format(date, 'DD. MM. YYYY');
        };
        MinuteSelectionStrategy.prototype.isEqual = function (a, b) {
            return this.previousStrategy().isEqual(a, b)
                && date_fns_1.getMinutes(a) === date_fns_1.getMinutes(b);
        };
        return MinuteSelectionStrategy;
    }(BaseStrategy));
    exports.MinuteSelectionStrategy = MinuteSelectionStrategy;
    function getFirstDayOfWeek(date) {
        return date_fns_1.startOfWeek(date, { weekStartsOn: 1 });
    }
});
